#!/usr/bin/perl -w

# StalkeesData v1.0
# Tony Flick (tony.flick[at)fyrmassociates.com)
# Justin Morehouse (justin.morehouse(at]gmail.com)

# This script allows a user to add a new twitter account to follow or to grab more messages from an existing stalkee

# The latest version is always available for download at http://www.fyrmassociates.com/tools.html

use CGI;
use HTML::Entities;
use Net::Twitter;
use Tie::File;
use Data::Dumper;
use POSIX;

my $MAXPAGES = 30;

my (@twitsFollow, $q, $ntREST, $ntSearch, $ntTV, $dataFile, $fileLines, $tzoffset, %rateLimit);

$q = new CGI();
$ntREST = new Net::Twitter->new(traits=>[qw/API::REST API::RateLimit API::Search/]);

print "Content-type: text/html\n\n";

if ($q->param("newStalkee") && $q->param("tzoffset")){

	$twitsFollow[0] = $q->param("newStalkee");
	my $tweetFile = "DataFiles/" . $twitsFollow[0] . ".csv";

	if (-e $tweetFile){
		print "This twitter acount, ", $twitsFollow[0], ", is already being stalked. Please use the existingStalkee method.";
		exit 0;
	}

	# Print user information to file
	open(INFOFILE, ">DataFiles/" . $twitsFollow[0] . ".dat");
	my $userInfo = $ntREST->show_user($twitsFollow[0]);
	print INFOFILE Dumper($userInfo);
	close(INFOFILE);

	# Set the offset for timezone purposes, Twitter's offset is unreliable, so need to set it manually for now
	$tzoffset = $q->param("tzoffset");
	$tzoffset *= 3600;

	# Calculate the number of pages of tweets; 
	# Divide their total number of tweets by the number of tweets on a page
	my $numberOfPages = ceil($userInfo->{statuses_count} / 20);
	if ($numberOfPages > $MAXPAGES){
		print "We could not download all of the status messages yet, due to Twitter's restrictions.<p>";
		print "There are currently " . ($numberOfPages - $MAXPAGES) . " pages of messages left. ";
		print "Please rerun this script once an hour until you have downloaded all messages. ";
		print "If you try to run this more frequently, you (or more appropriately your IP Address) risk being blacklisted by Twitter.";
	}
	else{
		print "The latest tweets for ", HTML::Entities::encode($twitsFollow[0]), " have been downloaded.";
	}

	# Download as many tweets as possible before getting cutoff by the limit
	my $pageCount = $numberOfPages;
	my @foStatuses;
	while (($pageCount > 0) && ($pageCount > ($numberOfPages-$MAXPAGES))){

		my $statuses = $ntREST->user_timeline({id=>$twitsFollow[0], count=>20, page=>$pageCount});
		my @foStatusTemp;
	
		# Print to the browser and file
		for my $status (@$statuses){

			# If the status message contains a comma, we need to put quotes around the entire status message
			if ($status->{text} =~ m/,/){
				my $tempStatusLine = $status->{id} . ",\"" . $status->{text} . "\"," . $status->{created_at} . "," . $tzoffset . "," . $pageCount;
				push(@foStatusTemp, $tempStatusLine);
			}
			else{
				my $tempStatusLine = $status->{id} . "," . $status->{text} . "," . $status->{created_at} . "," . $tzoffset . "," . $pageCount;
				push(@foStatustemp, $tempStatusLine);
			}
		} # Ends for my $status (@$statuses){
		unshift(@foStatuses, @foStatusTemp);

		$pageCount -= 1;

	} # Ends while (($pageCount < $numberOfPages) && ($pageCount < 50)){

	# Print to file
	open(OFILE, ">" . $tweetFile);
	for my $m (@foStatuses){
		print OFILE "$m\n";
	}
	close(OFILE);

} # Ends if ($q->param("newStalkee") && $q->param("tzoffset")){

# if the user wants to continue looking at an existing stalkee
elsif ($q->param("existingStalkee")){

	$twitsFollow[0] = $q->param("existingStalkee");
	my $fileName = "DataFiles/" . $twitsFollow[0] . ".csv";

	if (-e $fileName){

		tie @fileLines, 'Tie::File', $fileName; # || die ("Could not open $fileName<br>\n");
			
		my @tempTZ = parseCSV($fileLines[0]);
		$tzoffset = $tempTZ[scalar(@tempTZ) - 2];
		my $lastPage = $tempTZ[scalar(@tempTZ) - 1];
		my $lastID = $tempTZ[0];
		my @lastStatus = parseCSV($fileLines[0]);
		my $userInfo = $ntREST->show_user($twitsFollow[0]);
		my $numberOfPages = ceil($userInfo->{statuses_count} / 20);
		my $pageCount = $lastPage;

		while (($pageCount > 0) && ($pageCount > ($lastPage - $MAXPAGES))){

			my $statuses = $ntREST->user_timeline({id=>$twitsFollow[0], count=>20, page=>$pageCount,
								since_id=>$lastID});
	
			my @foStatusTemp;
	
			# Print to the browser and file
			for my $status (@$statuses){

				# If the status message contains a comma, we need to put quotes around the entire status message
				if ($status->{text} =~ m/,/){
					my $tempStatusLine = $status->{id} . ",\"" . $status->{text} . "\"," . $status->{created_at} . "," . $tzoffset . "," . $pageCount;
					push(@foStatusTemp, $tempStatusLine);
				}
				else{
					my $tempStatusLine = $status->{id} . "," . $status->{text} . "," . $status->{created_at} . "," . $tzoffset . "," . $pageCount;
					push(@foStatustemp, $tempStatusLine);
				}
			} # Ends for my $status (@$statuses){

			unshift(@foStatuses, @foStatusTemp);
			$pageCount -= 1;

		} # Ends while (($pageCount < $numberOfPages) && ($pageCount < 50)){

		unshift(@fileLines, @foStatuses);

		if ($lastPage > 1){
			print "We could not download all of the status messages yet, due to Twitter's restrictions.<p>";
			print "There are currently " . ($lastPage - 1) . " pages of messages left. ";
			print "Please rerun this script once an hour until you have downloaded all messages. ";
			print "If you try to run this more frequently, you (or more appropriately your IP Address) risk being blacklisted by Twitter.";
		}
		else{
			print "The latest tweets for ", HTML::Entities::encode($twitsFollow[0]), " have been downloaded.";
		}

	} # Ends if (-e $fileName){

	untie @fileLines;

} # ends elsif ($q->param("existingStalkee")){

	else{
		print "Please use and enter a value for the stalkee and tzoffset parameter, or the existingStalkee parameter . ";
		print "<p>Usage: energy_profiler.pl?newStalkee=&lt;twitter name&gt;&tzoffset=&lt;hours&gt;<br />";		
		print "<p>Usage: energy_profiler.pl?existingStalkee=&lt;twitter name&gt;<br />";
	}
#} # Ends the blacklisting if statement


# Parse a csv string into an array
sub parseCSV {
  my $text = shift;
  my @new = ();
  push (@new, $+) while $text =~ m{
          "([^\"\\]*(?:\\.[^\"\\]*)*)",?
          | ([^,]+),?
          | ,
      }gx;
  push (@new, undef) if substr($text, -1, 1) eq ",";
  return @new;
}

